==> prayer-1.0.12.tar.gz: prayer-1.0.12/prayer/cmd_abook_list.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_abook_list.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
html_button_single_table(struct session *s,
                         struct buffer *b,
                         char *cmd,
                         unsigned long offset, char *img, char *alt)
{
    bputs(b, "<td>");
    html_session_bprintf(s, b, "<a href=\"", "\">", "%s/%lu", cmd, offset);
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>" CRLF);
}

static void
cmd_abook_list_hdr(struct session *session, struct buffer *b,
                   unsigned long offset, unsigned long count,
                   unsigned long per_page)
{
    struct config_theme *theme = session->theme_main;
    BOOL use_icons = session->user_agent->use_icons;
    unsigned long page;
    unsigned long pages;

    if (count > 0) {
        page = (offset / per_page) + 1;
        pages = ((count - 1) / per_page) + 1;
    } else {
        page = 0;
        pages = 0;
    }

    bputs(b, "<table width=\"100%\" cellspacing=0 cellpadding=0>" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table><tr>" CRLF);

    if (page > 1) {
        html_button_single_table(session, b, "abook_list", 1,
                                 "leftleft.gif", "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");

        html_button_single_table(session, b, "abook_list",
                                 (offset - per_page), "left.gif",
                                 "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        html_button_single_table(session, b, "abook_list",
                                 (offset + per_page), "right.gif", "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_single_table(session, b, "abook_list", count,
                                 "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }

    if (session->draft->have_draft)
        html_form_table_submit(b, "sub_add_address",
                               "Add marked to draft");

    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width =\"33%\" align=\"center\">");
    if (count != 1)
        bprintf(b, "<strong>Addressbook (%lu entries)</strong>", count);
    else
        bprintf(b, "<strong>Addressbook (1 entry)</strong>");

    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"34%\" align=\"right\"><table><tr>" CRLF);
    html_textlink_session_table(session, b, "abook_add", "Add");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_xfer", "Transfer");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_search", "Search");
    bputs(b, "</tr></table></tr></table>" CRLF);
}

static void
cmd_abook_list_footer(struct session *session, struct buffer *b,
                      unsigned long offset, unsigned long count,
                      unsigned long per_page)
{
    struct config_theme *theme = session->theme_main;
    BOOL use_icons = session->user_agent->use_icons;
    unsigned long page;
    unsigned long pages;

    if (count > 0) {
        page = (offset / per_page) + 1;
        pages = ((count - 1) / per_page) + 1;
    } else {
        page = 0;
        pages = 0;
    }

    bputs(b, "<table width=\"100%\" cellspacing=0 cellpadding=0>" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table><tr>" CRLF);

    if (page > 1) {
        html_button_single_table(session, b, "abook_list", 1,
                                 "leftleft.gif", "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");

        html_button_single_table(session, b, "abook_list",
                                 (offset - per_page), "left.gif",
                                 "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        html_button_single_table(session, b, "abook_list",
                                 (offset + per_page), "right.gif", "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_single_table(session, b, "abook_list", count,
                                 "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }

    if (session->draft->have_draft)
        html_form_table_submit(b, "sub_add_address",
                               "Add marked to draft");

    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"center\">");
    bputs(b, "<table><tr>");
    bputs(b, "<td>Page:</td>");
    bprintf(b, "<td><input name=\"page\" size=4 value=\"%lu\"></td>",
            page);
    bprintf(b, "<td>/%lu</td>", pages);
    bputs(b,
          "<td><input type=\"submit\" name=\"sub_page\" value=\"Go\"></td>");
    bputs(b, "</tr></table>");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"right\"><table><tr>" CRLF);

    html_textlink_session_table(session, b, "abook_add", "Add");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_xfer", "Transfer");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_search", "Search");
    bputs(b, "</tr></table></tr></table>" CRLF);
}

static void
cmd_abook_list_single(struct session *session, struct buffer *b,
                      struct abook_entry *abe, unsigned long offset,
                      BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    struct request *request = session->request;
    struct pool *pool = request->pool;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;


    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td><strong>%lu.</strong></td>" CRLF, offset);

    html_session_bprintf(session, b, "<td><a href=\"", "\">",
                         ("abook_update?alias=%s&amp;name=%s&amp;"
                          "comment=%s&amp;email=%s"),
                         string_url_encode(pool, abe->alias),
                         string_url_encode(pool, abe->name),
                         string_url_encode(pool, abe->comment),
                         string_url_encode(pool, abe->email));

    if (abe->alias && abe->alias[0])
        html_quote_string(b, abe->alias);
    else
        bputs(b, "&nbsp;" CRLF);

    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td>");
    if (abe->name && abe->name[0])
        html_quote_string(b, abe->name);
    else
        bputs(b, "&nbsp;" CRLF);
    bputs(b, "</td>");

    bputs(b, "<td>");
    if (abe->comment && abe->comment[0])
        html_quote_string(b, abe->comment);
    else
        bputs(b, "&nbsp;" CRLF);

    bputs(b, "</td>");

    bputs(b, "<td>");
    if (abe->email && abe->email[0])
        html_quote_string(b, abe->email);
    else
        bputs(b, "&nbsp;");

    bputs(b, "</td>" CRLF);

    if (session->draft->have_draft) {
        /* Add links with existing draft */

        bputs(b, "<td>");
        bprintf(b, "<input type=\"checkbox\" name=\"to_%lu\">To" CRLF,
                offset);
        bprintf(b, "<input type=\"checkbox\" name=\"cc_%lu\">Cc" CRLF,
                offset);
        bprintf(b, "<input type=\"checkbox\" name=\"bcc_%lu\">Bcc" CRLF,
                offset);
        bputs(b, "</td>");
    } else {
        /* Separate as may need to select role */
        html_session_bprintf(session, b, "<td><a href=\"",
                             "\">Compose</a></td>" CRLF,
                             "abook_compose/%s/%s",
                             string_canon_encode(pool, abe->name),
                             string_canon_encode(pool, abe->email));
    }
    bputs(b, "</tr>" CRLF);
}

static void process_form(struct session *session)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct abook *abook = options->abook;
    struct request *request = session->request;
    struct draft *draft = session->draft;
    unsigned long offset;
    unsigned long count;
    struct abook_entry *abe;
    char *key;

    request_decode_post(request);

    if (!assoc_lookup(request->form, "sub_add_address")) {
        char *pstr = assoc_lookup(request->form, "page");
        unsigned long page = pstr ? atoi(pstr) : 0;
        unsigned long size = list_length(abook->list);
        unsigned long pages = (size / prefs->abook_per_page) + 1;

        if ((page > 0) && (page <= pages))
            abook->current = (page - 1) * prefs->abook_per_page + 1;

        if (abook->current > size)
            abook->current = size;
        return;
    }

    assoc_scan_reset(request->form);

    count = 0;
    while (assoc_scan_next(request->form, &key, NIL)) {
        if (!strncmp(key, "to_", strlen("to_"))) {
            offset = atoi(key + strlen("to_")) - 1;

            if ((abe = abook_lookup_byoffset(abook, offset))) {
                draft_add_to(draft,
                             abook_entry_to_string(request->pool, abe));
                count++;
            }
        } else if (!strncmp(key, "cc_", strlen("cc_"))) {
            offset = atoi(key + strlen("to_")) - 1;

            if ((abe = abook_lookup_byoffset(abook, offset))) {
                draft_add_cc(draft,
                             abook_entry_to_string(request->pool, abe));
                count++;
            }
        } else if (!strncmp(key, "bcc_", strlen("bcc_"))) {
            offset = atoi(key + strlen("bcc_")) - 1;

            if ((abe = abook_lookup_byoffset(abook, offset))) {
                draft_add_bcc(draft,
                              abook_entry_to_string(request->pool, abe));
                count++;
            }
        }
    }
    if (count != 1)
        session_message(session, "Added %lu addresses to draft", count);
    else
        session_message(session, "Added 1 address to draft");
}

void cmd_abook_list(struct session *session)
{
    struct config_theme *theme = session->theme_main;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct options *options = session->options;
    struct abook *abook = options->abook;
    struct prefs *prefs = options->prefs;
    struct abook_entry *abe;
    unsigned long offset;
    unsigned long first, last, count;
    BOOL shaded;

    /* Save options if anything changed */
    if (options->save)
        session_streams_save_options(session);

    if (request->method == POST)
        process_form(session);
    else if (request->argc == 2)
        abook->current = atoi(request->argv[1]);        /* Move to page */

    /* Record last command for compose/cancel and friends */
    session->compose_parent_cmd = "abook_list";

    /* Title bar */
    html_start(session, b, "Addressbook");

    /* Print toolbar */
    html_banner_toolbar(session, b, "abook_list", 0);

    /* Work out if "current" entry still exists. If not, make the last
     * address on the list current. */
    if ((count = list_length(abook->list)) > 0) {
        if (abook->current > count)
            abook->current = count;
        else if (abook->current == 0)
            abook->current = 1;
    } else
        abook->current = 0;

    html_form_start(session, b, "POST", "abook_list");

    cmd_abook_list_hdr(session, b, abook->current, count,
                       prefs->abook_per_page);

    if (count == 0) {
        bputs(b, "Address book is empty");

        html_end(b);
        response_html(request, 200);    /* Success */
        return;
    }

    bputs(b, "<table width=\"100%\">" CRLF);

    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"3%\">#</td>" CRLF);
    bputs(b, "<td width=\"10%\">Alias</td>" CRLF);
    bputs(b, "<td width=\"15%\">Name</td>" CRLF);
    bputs(b, "<td width=\"17%\">Comment</td>" CRLF);
    bputs(b, "<td width=\"40%\">Address(es)</td>" CRLF);
    bputs(b, "<td width=\"15%\">&nbsp</td>" CRLF);
    bputs(b, "</tr>" CRLF);

    /* Calculate first and last entry on page. */
    offset = abook->current;

    if (offset >= prefs->abook_per_page)
        first = (offset - ((offset - 1) % prefs->abook_per_page));
    else
        first = 1;

    last = (((first + (prefs->abook_per_page - 1)) <= count)
            ? (first + (prefs->abook_per_page - 1)) : count);

    offset = first;
    abe = abook_lookup_byoffset(abook, offset - 1);
    shaded = T;

    while (abe && (offset <= last)) {
        cmd_abook_list_single(session, b, abe, offset, shaded);

        shaded = (shaded) ? NIL : T;
        abe = abe->next;
        offset++;
    }
    bputs(b, "</table>" CRLF);

    cmd_abook_list_footer(session, b,
                          abook->current, count, prefs->abook_per_page);

    html_form_end(b);
    html_end(b);
    response_html(request, 200);        /* Success */
}

==> prayer-1.0.16.tar.gz: prayer-1.0.16/prayer/cmd_abook_list.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_abook_list.c,v 1.19 2005/10/05 10:56:42 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
html_button_single_table(struct session *s,
                         struct buffer *b,
                         char *cmd,
                         unsigned long offset, char *img, char *alt)
{
    bputs(b, "<td>");
    html_session_bprintf(s, b, "<a href=\"", "\">", "%s/%lu", cmd, offset);
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>" CRLF);
}

static void
cmd_abook_list_hdr(struct session *session, struct buffer *b,
                   unsigned long offset, unsigned long count,
                   unsigned long per_page)
{
    struct config_theme *theme = session->theme_main;
    BOOL use_icons = session->user_agent->use_icons;
    unsigned long page;
    unsigned long pages;

    if (count > 0) {
        page = ((offset - 1) / per_page) + 1;
        pages = ((count - 1) / per_page) + 1;
    } else {
        page = 0;
        pages = 0;
    }

    bputs(b, "<table width=\"100%\" cellspacing=0 cellpadding=0>" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table><tr>" CRLF);

    if (page > 1) {
        html_button_single_table(session, b, "abook_list", 1,
                                 "leftleft.gif", "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");

        html_button_single_table(session, b, "abook_list",
                                 (offset - per_page), "left.gif",
                                 "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        html_button_single_table(session, b, "abook_list",
                                 (offset + per_page), "right.gif", "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_single_table(session, b, "abook_list", count,
                                 "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }
    bputs(b, "<td>");
    html_form_submit(b, "sub_remove", "Remove Marked Entries");
    bputs(b, "</td>");
    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width =\"33%\" align=\"center\">");
    if (count != 1)
        bprintf(b, "<strong>Addressbook (%lu entries)</strong>", count);
    else
        bprintf(b, "<strong>Addressbook (1 entry)</strong>");

    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"34%\" align=\"right\"><table><tr>" CRLF);
    html_textlink_session_table(session, b, "abook_add", "Add");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_xfer", "Transfer");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_search", "Search");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
}

static void
cmd_abook_list_footer(struct session *session, struct buffer *b,
                      unsigned long offset, unsigned long count,
                      unsigned long per_page)
{
    struct config_theme *theme = session->theme_main;
    BOOL use_icons = session->user_agent->use_icons;
    unsigned long page;
    unsigned long pages;

    if (count > 0) {
        page = ((offset -1) / per_page) + 1;
        pages = ((count - 1) / per_page) + 1;
    } else {
        page = 0;
        pages = 0;
    }

    bputs(b, "<table width=\"100%\" cellspacing=0 cellpadding=0>" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table><tr>" CRLF);

    if (page > 1) {
        html_button_single_table(session, b, "abook_list", 1,
                                 "leftleft.gif", "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");

        html_button_single_table(session, b, "abook_list",
                                 (offset - per_page), "left.gif",
                                 "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        html_button_single_table(session, b, "abook_list",
                                 (offset + per_page), "right.gif", "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_single_table(session, b, "abook_list", count,
                                 "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }
    bputs(b, "<td>");
    html_form_submit(b, "sub_remove", "Remove Marked Entries");
    bputs(b, "</td>");
    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"center\">");
    bputs(b, "<table><tr>");
    bputs(b, "<td>Page:</td>");
    bprintf(b, "<td><input name=\"page\" size=4 value=\"%lu\"></td>",
            page);
    bprintf(b, "<td>/%lu</td>", pages);
    bputs(b,
          "<td><input type=\"submit\" name=\"sub_page\" value=\"Go\"></td>");
    bputs(b, "</tr></table>");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"right\"><table><tr>" CRLF);

    html_textlink_session_table(session, b, "abook_add", "Add");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_xfer", "Transfer");
    bputs(b, "<td>|</td>");
    html_textlink_session_table(session, b, "abook_search", "Search");
    bputs(b, "</tr></table></tr></table>" CRLF);
}

/* Similar to string_canon_encode(), but encodes _all_ non alnum characters.
 * '.' (and possibly other characters) cause grief in HTML form checkboxes */

static void
print_alias_canon_encoding(struct buffer *b, char *s)
{
    static char hex[] = "0123456789abcdef";
    char c;

    if (!s)
        return;

    while ((c=*s++)) {
        if (!Uisalnum(c)) {
            bputc(b, '*');
            bputc(b, hex[c >> 4]);
            bputc(b, hex[c & 15]);
        } else
            bputc(b, c);
    }
}

static void
cmd_abook_list_single(struct session *session, struct buffer *b,
                      struct abook_entry *abe, unsigned long offset,
                      BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    struct request *request = session->request;
    struct pool *pool = request->pool;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td><strong>%lu.</strong></td>" CRLF, abe->position);

    bputs(b, "<td>");
    bputs(b, "<input type=\"checkbox\" name=\"remove_");
    print_alias_canon_encoding(b, abe->alias);
    bputs(b, "\"></td>"CRLF);

    html_session_bprintf(session, b, "<td><a href=\"", "\">"CRLF,
                         ("abook_update?alias=%s&amp;name=%s&amp;"
                          "comment=%s&amp;email=%s"),
                         string_url_encode(pool, abe->alias),
                         string_url_encode(pool, abe->name),
                         string_url_encode(pool, abe->comment),
                         string_url_encode(pool, abe->email));

    if (abe->alias && abe->alias[0])
        html_quote_string(b, abe->alias);
    else
        bputs(b, "&nbsp;" CRLF);

    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td>");
    if (abe->name && abe->name[0])
        html_quote_string(b, abe->name);
    else
        bputs(b, "&nbsp;" CRLF);
    bputs(b, "</td>");

    bputs(b, "<td>");
    if (abe->comment && abe->comment[0])
        html_quote_string(b, abe->comment);
    else
        bputs(b, "&nbsp;" CRLF);

    bputs(b, "</td>");

    bputs(b, "<td>");
    if (abe->email && abe->email[0])
        html_quote_string(b, abe->email);
    else
        bputs(b, "&nbsp;");

    bputs(b, "</td>" CRLF);

    if (session->draft->have_draft) {
        /* Add links with existing draft */

        bputs(b, "<td>");
        bprintf(b, "<input type=\"checkbox\" name=\"to_%lu\">To" CRLF,
                offset);
        bprintf(b, "<input type=\"checkbox\" name=\"cc_%lu\">Cc" CRLF,
                offset);
        bprintf(b, "<input type=\"checkbox\" name=\"bcc_%lu\">Bcc" CRLF,
                offset);
        bputs(b, "</td>");
    } else {
        /* Separate as may need to select role */
        html_session_bprintf(session, b, "<td><a href=\"",
                             "\">Compose</a></td>" CRLF,
                             "abook_compose/%s/%s",
                             string_canon_encode(pool, abe->name),
                             string_canon_encode(pool, abe->email));
    }
    bputs(b, "</tr>" CRLF);
}

static void process_remove(struct session *session)
{
    struct options *options = session->options;
    struct abook *abook = options->abook;
    struct request *request = session->request;
    struct pool *pool = request->pool;
    unsigned long count;
    char *key, *tmp;

    assoc_scan_reset(request->form);

    count = 0;
    while (assoc_scan_next(request->form, &key, NIL)) {
        if (strncmp(key, "remove_", strlen("remove_")) != 0)
            continue;

        tmp = pool_strdup(pool, key + strlen("remove_"));
        string_canon_decode(tmp);

        abook_remove(abook, tmp);
        count++;
    }

    if (count > 0)
        session->options->save = T;

    if (count != 1)
        session_message(session,
                        "Removed %lu entries from addressbook", count);
    else
        session_message(session, "Added 1 entry from addressbook");
}

static void process_add_address(struct session *session)
{
    struct options *options = session->options;
    struct abook *abook = options->abook;
    struct request *request = session->request;
    struct draft *draft = session->draft;
    unsigned long offset;
    unsigned long count;
    struct abook_entry *abe;
    char *key;

    assoc_scan_reset(request->form);

    count = 0;
    while (assoc_scan_next(request->form, &key, NIL)) {
        if (!strncmp(key, "to_", strlen("to_"))) {
            offset = atoi(key + strlen("to_")) - 1;

            if ((abe = abook_lookup_sorted_byoffset(abook, offset))) {
                draft_add_to(draft,
                             abook_entry_to_string(request->pool, abe));
                count++;
            }
        } else if (!strncmp(key, "cc_", strlen("cc_"))) {
            offset = atoi(key + strlen("to_")) - 1;

            if ((abe = abook_lookup_sorted_byoffset(abook, offset))) {
                draft_add_cc(draft,
                             abook_entry_to_string(request->pool, abe));
                count++;
            }
        } else if (!strncmp(key, "bcc_", strlen("bcc_"))) {
            offset = atoi(key + strlen("bcc_")) - 1;

            if ((abe = abook_lookup_sorted_byoffset(abook, offset))) {
                draft_add_bcc(draft,
                              abook_entry_to_string(request->pool, abe));
                count++;
            }
        }
    }
    if (count != 1)
        session_message(session, "Added %lu addresses to draft", count);
    else
        session_message(session, "Added 1 address to draft");
}

void cmd_abook_list(struct session *session)
{
    struct config *config = session->config;
    struct config_theme *theme = session->theme_main;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct options *options = session->options;
    struct abook *abook = options->abook;
    struct prefs *prefs = options->prefs;
    struct lookup *lookup = session->lookup;
    struct abook_entry *abe;
    unsigned long offset;
    unsigned long first, last, count;
    BOOL shaded;

    /* Save options if anything changed */
    if (options->save)
        session_streams_save_options(session);

    if (request->method == POST) {
        request_decode_post(request);

        if (assoc_lookup(request->form, "sub_remove")) {
            process_remove(session);
        } else if (assoc_lookup(request->form, "sub_add_address")) {
            process_add_address(session);
        } else {
            char *pstr = assoc_lookup(request->form, "page");
            unsigned long page = pstr ? atoi(pstr) : 0;
            unsigned long size = list_length(abook->list);
            unsigned long pages = (size / prefs->abook_per_page) + 1;

            if ((page > 0) && (page <= pages))
                abook->current = (page - 1) * prefs->abook_per_page + 1;

            if (abook->current > size)
                abook->current = size;
        }
    } else if (request->argc == 2)
        abook->current = atoi(request->argv[1]);        /* Move to page */

    /* Record last command for compose/cancel and friends */
    if (!session->draft->have_draft)
        session->compose_parent_cmd = "abook_list";

    /* Title bar */
    html_start(session, b, "Addressbook");

    /* Print toolbar */
    html_banner_toolbar(session, b, "abook_list", 0);

    if (lookup_local_available(lookup) || lookup_ldap_available(lookup)) {
        /* Link into directory */
        html_form_start(session, b, "POST", "abook_lookup");
        bputs(b, "<table width=\"100%\"><tr><td align=\"right\">"CRLF);
        bputs(b, "<table><tr>"CRLF);
        bputs(b, "<td>Surname:</td>");
        html_form_table_field(b, "", "query", "", 20);
        if (lookup_local_available(lookup)) {
            char *s;

            if (config->login_service_name)
                s = pool_printf(request->pool, "Search %s Users",
                                config->login_service_name);
            else
                s = "Search Local Users";

            html_form_table_submit(b, "sub_local", s);
        }
        if (lookup_ldap_available(lookup))
            html_form_table_submit(b, "sub_ldap",  "Search Directory");
        bputs(b, "</tr></table></td></tr></table>"CRLF);
        html_form_end(b);
    }

    /* Work out if "current" entry still exists. If not, make the last
     * address on the list current. */
    if ((count = list_length(abook->list)) > 0) {
        if (abook->current > count)
            abook->current = count;
        else if (abook->current == 0)
            abook->current = 1;
    } else
        abook->current = 0;

    html_form_start(session, b, "POST", "abook_list");

    cmd_abook_list_hdr(session, b, abook->current, count,
                       prefs->abook_per_page);

    if (count == 0) {
        bputs(b, "Address book is empty");

        html_end(b);
        response_html(request, 200);    /* Success */
        return;
    }

    bputs(b, "<table width=\"100%\">" CRLF);

    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"3%\">");
    html_textlink_session(session, b, "abook_sort/order", "#");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"2%\" align=\"center\">");
    bputs(b, "<input type=\"checkbox\" name=\"noop\" checked>");
    bputs(b, "</td>");

    bputs(b, "<td width=\"8%\">");
    html_textlink_session(session, b, "abook_sort/alias", "Alias");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"15%\">");
    html_textlink_session(session, b, "abook_sort/name", "Name");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"17%\">");
    html_textlink_session(session, b, "abook_sort/comment", "Comment");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"40%\">");
    html_textlink_session(session, b, "abook_sort/email", "Address(es)");
    bputs(b, "</td>" CRLF);

    if (session->draft->have_draft) {
        bputs(b, "<td width=\"15%\" align=\"center\">" CRLF);
        html_form_submit(b, "sub_add_address", "Add marked to draft");
        bputs(b, "</td>" CRLF);
    } else
        bputs(b, "<td width=\"15%\">&nbsp</td>" CRLF);

    bputs(b, "</tr>" CRLF);

    /* Calculate first and last entry on page. */
    offset = abook->current;

    if (offset >= prefs->abook_per_page)
        first = (offset - ((offset - 1) % prefs->abook_per_page));
    else
        first = 1;

    last = (((first + (prefs->abook_per_page - 1)) <= count)
            ? (first + (prefs->abook_per_page - 1)) : count);

    abook_sort(abook);

    offset = first;
    shaded = T;

    while (offset <= last) {
        if (!(abe = abook_lookup_sorted_byoffset(abook, offset - 1)))
            break;

        cmd_abook_list_single(session, b, abe, offset, shaded);

        shaded = (shaded) ? NIL : T;
        offset++;
    }
    bputs(b, "</table>" CRLF);

    cmd_abook_list_footer(session, b,
                          abook->current, count, prefs->abook_per_page);

    html_form_end(b);
    html_end(b);
    response_html(request, 200);        /* Success */
}
